import logging
import os
import random
import string
import imageio
import datetime
import json

from multiprocessing import Pool
from PIL import Image, ImageDraw, ImageFont
from pyrest.assets import assetsDir

from pyrest.utils.logger.logger import Logger


class TimelapseFactory:
    @staticmethod
    def factory(type):
        if type == "GIF":
            return GifTimeLapse()
        else:
            raise Exception("Cannot create timelapse of type " + type)


class Timelapse(object):

    def __init__(self):
        self.images = list()
        self.tmp_dir = "/tmp/"
        self.logging = Logger('timelapse.log')


    def addWMSImage(self, url):
        self.images.append(url)

    def fetchImages(self):
        p = Pool(5)
        self.logging.debug("Fetching images %s" % ','.join(json.dumps(image) for image in self.images))
        imageList = p.map(fetchImage, self.images)
        p.close()
        p.join()
        return imageList

def fetchImage(image):
    return (image["text"], Image.fromarray(imageio.imread(uri=image["url"], format="PNG" )))

class GifTimeLapse(Timelapse):

    def create(self, application, delay=1, loop=0):
        save_dir = os.path.join(self.tmp_dir, ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)))
        self.logging.debug("Creating temporaray directory at %s " % (save_dir))
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        save_dir = save_dir + "/pygif.gif"
        self.logging.debug("Fetching %s images" % (str(len(self.images))))
        frames = list()
        for frame in self.fetchImages():
            image = self.addLogo(image=frame[1], application=application)
            image = self.addText(image=frame[1], text=frame[0])
            frames.append(image)
        self.logging.debug("Creating gif image with delay of %s and looping set to %s" % (str(delay), str(loop)))
        frames[0].save(os.path.join(save_dir), save_all=True, append_images=frames[1:], duration=delay, loop=loop, optimize=False)
        return save_dir

    def addText(self, image, text):

        font = ImageFont.truetype("%s/fonts/OpenSans-Bold.ttf" % assetsDir, 26)

        text_size = font.getsize(text)
        background_size = (text_size[0] + 30, text_size[1] + 25)

        textComponent = Image.new('RGBA', image.size, (0, 0, 0, 0))
        draw = ImageDraw.Draw(textComponent)
        draw.rectangle(((0, 0), (background_size[0], background_size[1])), fill=(255, 255, 255, 204))
        draw.text((15, 10), text, (0, 0, 0), font=font)

        # put button on source image in position (0, 0)
        image.paste(textComponent, (0,0), textComponent)
        return image

    def addLogo(self, image, application):

        logoDir = '%s/logos/%s.png' % (assetsDir , application)
        self.logging.debug("Adding logo at path %s to the frame" % logoDir)

        logo = Image.open(logoDir).convert("RGBA")

        # resize logo
        wsize = int(min(image.size[0], image.size[1]) * 0.25)
        wpercent = (wsize / float(logo.size[0]))
        hsize = int((float(logo.size[1]) * float(wpercent)))

        simage = logo.resize((wsize, hsize))
        mbox = image.getbbox()
        sbox = simage.getbbox()

        # right bottom corner
        box = (mbox[2] - sbox[2] - 10, mbox[3] - sbox[3] -10)
        image.paste(simage, box, simage)
        return image

